package fr.doap.trash;

import fr.doap.slog.*;

import java.util.concurrent.*;
import java.util.concurrent.locks.*;
import java.util.function.*;

public class ProcessingQueue<T> {
	
	public ProcessingQueue(Consumer<T> aProcessor) {
		mProcessor = aProcessor;
		mThread = new Thread(() -> {
			mLock.lock();
			L.t("Starting ProcessingQueue {}", mThread);
			try {
				while (mStillRunning) {
					T lT = mQueue.poll(50, TimeUnit.MILLISECONDS);
					if (lT != null)	process(lT);
				}
			} catch (Exception aE) {}
			L.t("Flushing ProcessingQueue {}", mThread);
			mQueue.forEach(aT -> process(aT));
			L.t("Exiting ProcessingQueue {}", mThread);
			mLock.unlock();
		});
		mThread.setDaemon(true);
		mThread.start();
	}

	public boolean add(T aT) {
		if (mStillRunning) {
			L.t("Adding {} to ProcessingQueue {}", aT, mThread);
			mQueue.add(aT);
			return true;
		}
		return false;
	}
	
	public void process(T aT) {
		L.t("Processing {} with ProcessingQueue {}", aT, mThread);
		mProcessor.accept(aT);
	}
	
	public Lock stop() {
		if ((mThread != null)&&(mThread.isAlive())) {
			L.t("Stopping ProcessingQueue {}", mThread);
			mStillRunning = false;
		}
		return mLock;
	}
	
	Thread				mThread;
	BlockingQueue<T>	mQueue = new LinkedBlockingQueue<>();
	Consumer<T>			mProcessor;
	boolean				mStillRunning = true;
	ReentrantLock		mLock = new ReentrantLock();
}
