package fr.doap.jdb;

import fr.doap.slog.*;
import org.junit.jupiter.api.*;

import java.sql.*;
import java.util.*;

@TestMethodOrder(MethodOrderer.MethodName.class)
public class DBWrapTest {
	
	@BeforeAll
	static void setUp() {
		if (sDB == null) {
			L.i("Creating test db");
			sDB=new DBWrap("jdbc:h2:~/test", "sa", "").setSQLTypeResolver(aS -> "VARCHAR");
		}
	}
	static DBWrap sDB;
	
	@Test
	public void test1() {
		Map<String, Object> lMap = new HashMap<>();
		lMap.put("key1", "val1");
		lMap.put("key1.1", "val1.1");
		lMap.put("key2", "val2");
		lMap.put("_JID_", 1);
		sDB.insert(lMap);
	}
	
	@Test
	public void test2() {
		Map<String, Object> lMap = new HashMap<>();
		lMap.put("key1", "val1bis");
		lMap.put("key3", "val3");
		lMap.put("_JID_", 2);
		sDB.insert(lMap);
	}
	
	@Test
	public void test3() {
		try {
			ResultSet lRS = sDB._query("SELECT * FROM OBJECTS").get();
			String lCols[] = null;
			while (lRS.next()) {
				if (lCols == null) {
					lCols = new String[lRS.getMetaData().getColumnCount()];
					for (int i=0; i<lCols.length; i++) lCols[i]=lRS.getMetaData().getColumnName(i+1);
				}
				List<String> lRes = new ArrayList<>();
				for (int i=0; i<lCols.length; i++) lRes.add(lCols[i]+":"+lRS.getObject(i+1));
				L.i(String.join(" / ", lRes));
			}
		} catch (Exception aE) { L.w(aE); }
	}
	
	@Test
	public void test4() {
		sDB.query("SELECT * FROM OBJECTS").forEach(aKV -> L.i(aKV.toString()));
	}
	
	@Test
	public void test5() {
		sDB.query("SELECT MAX(_JID_)+1 AS COUNT FROM OBJECTS").forEach(aKV -> L.i(aKV.toString()));
	}
	
	@AfterAll
	static void closeUp() {
		if (sDB != null) {
			L.i("deleting test db: ");
			try {
				sDB.exec("DROP ALL OBJECTS DELETE FILES");
				L.i("DONE");
			} catch(Exception aE) {
				L.i(aE.getLocalizedMessage());
				aE.printStackTrace();
			}
		}
	}
}
